Namespace Algorithms

    ''' <summary>
    ''' Class representing a tuple of two values.
    ''' </summary>
    ''' <typeparam name="T1">Type of first value.</typeparam>
    ''' <typeparam name="T2">Type of second value.</typeparam>
    Public Class Pair(Of T1 As IComparable, T2 As IComparable)
        Public V1 As T1
        Public V2 As T2

        Public Sub New(ByVal v1 As T1, ByVal v2 As T2)
            Me.V1 = v1
            Me.V2 = v2
        End Sub
    End Class

    Public Module Lists

        ''' <summary>
        ''' Shuffles a list of values.
        ''' </summary>
        ''' <typeparam name="T">Type of the objects in list.</typeparam>
        ''' <typeparam name="L">Type of the list.</typeparam>
        ''' <param name="list">Input list.</param>
        ''' <returns>Returns a list which contains the shuffled values.</returns>
        Public Function Shuffle(Of T, L As {IList(Of T), New})(ByVal list As L) As L
            Dim ret As New L
            Dim rnd As New Random()

            While list.Count > 0
                ret.Add(list.Item(rnd.Next(0, list.Count)))
            End While

            Return ret
        End Function

        ''' <summary>
        ''' Shuffles a list of values.
        ''' </summary>
        ''' <typeparam name="T">Type of the objects in list.</typeparam>
        ''' <param name="list">Input list.</param>
        ''' <returns>Returns a LIFO stack which contains the shuffled values.</returns>
        Public Function Shuffle(Of T)(ByVal list As List(Of T)) As Stack(Of T)
            Dim ret As New Stack(Of T)
            Dim rnd As New Random()

            While list.Count > 0
                Dim index As Integer = rnd.Next(0, list.Count)
                ret.Push(list.Item(index))
                list.RemoveAt(index)
            End While

            Return ret
        End Function

    End Module

End Namespace