Imports Microsoft.VisualBasic

Public Class MainDialog

    Private WithEvents m_game As MinesweeperGame

    Private Sub aboutToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles aboutToolStripMenuItem.Click
        AboutBox.ShowDialog()
    End Sub

    Private Sub exitToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles exitToolStripMenuItem.Click
        Me.Close()
    End Sub

    Private Sub newToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles newToolStripMenuItem.Click
        Me.m_game.BeginGame()
    End Sub

    Private Sub beginnerToolStripItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles beginnerToolStripItem.Click
        ShowGameTypeInMenu(sender)
        Me.m_game.BeginGame(MinesweeperGame.FixedConfigurations.Beginner)
    End Sub

    Private Sub advancedToolStripItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles advancedToolStripItem.Click
        ShowGameTypeInMenu(sender)
        Me.m_game.BeginGame(MinesweeperGame.FixedConfigurations.Advanced)
    End Sub

    Private Sub expertToolStripItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles expertToolStripItem.Click
        ShowGameTypeInMenu(sender)
        Me.m_game.BeginGame(MinesweeperGame.FixedConfigurations.Expert)
    End Sub

    Private Sub ShowGameTypeInMenu(ByVal sender As Object)
        Me.beginnerToolStripItem.Checked = False
        Me.advancedToolStripItem.Checked = False
        Me.expertToolStripItem.Checked = False
        Me.customToolStripItem.Checked = False
        DirectCast(sender, ToolStripMenuItem).Checked = True
    End Sub

    Private Sub m_game_Clicked(ByVal sender As MinesweeperGame) Handles m_game.Clicked
        Me.NumberOfMinesLabel.Text = sender.MinesLeft.ToString()
    End Sub

    Private Sub m_game_GameLost(ByVal sender As MinesweeperGame) Handles m_game.GameLost
        Me.newGameButton.Text = ":'("
    End Sub

    Private Sub m_game_GameStarted(ByVal sender As MinesweeperGame) Handles m_game.GameStarted
        Me.newGameButton.Text = "New"
        Me.NumberOfMinesLabel.Text = sender.MinesLeft.ToString()
    End Sub

    Private Sub m_game_GameWon(ByVal sender As MinesweeperGame) Handles m_game.GameWon
        Dim showHighscore As Boolean = False

        Me.newGameButton.Text = ":-)"

        Select Case True
            Case Me.beginnerToolStripItem.Checked
                If sender.ElapsedTime < My.Settings.BestBeginnerTime Then
                    Dim name As String = InputBox("New record. Please enter your name:", "Congratulations!")
                    My.Settings.BestBeginnerTime = sender.ElapsedTime
                    My.Settings.BestBeginner = name
                    My.Settings.Save()
                    showHighscore = True
                End If
            Case Me.advancedToolStripItem.Checked
                If sender.ElapsedTime < My.Settings.BestAdvancedTime Then
                    Dim name As String = InputBox("New record. Please enter your name:", "Congratulations!")
                    My.Settings.BestAdvancedTime = sender.ElapsedTime
                    My.Settings.BestAdvanced = name
                    My.Settings.Save()
                    showHighscore = True
                End If
            Case Me.expertToolStripItem.Checked
                If sender.ElapsedTime < My.Settings.BestExpertTime Then
                    Dim name As String = InputBox("New record. Please enter your name:", "Congratulations!")
                    My.Settings.BestExpertTime = sender.ElapsedTime
                    My.Settings.BestExpert = name
                    My.Settings.Save()
                    showHighscore = True
                End If
        End Select

        If showHighscore Then HighscoreDialog.ShowDialog()
    End Sub

    Private Sub m_game_RequestContainerResize(ByVal sender As MinesweeperGame, ByVal e As RequestContainerResizeEventArgs) Handles m_game.RequestContainerResize
        Me.Width = Me.playField.Left * 2 + e.Size.Width + Me.playField.Margin.Left + Me.playField.Margin.Right
        Me.Height = Me.playField.Top + e.Size.Height + Me.playField.Margin.Top + Me.playField.Margin.Bottom + Me.MainMenu.Height
        Me.newGameButton.Left = Me.controlPanel.Width \ 2 - Me.newGameButton.Width \ 2
    End Sub

    Private Sub m_game_Tick(ByVal sender As MinesweeperGame, ByVal e As TickEventArgs) Handles m_game.Tick
        Me.ElapsedTimeLabel.Text = e.ElapsedTime.ToString()
    End Sub

    Private Sub newGameButton_Click(ByVal sender As Object, ByVal e As EventArgs) Handles newGameButton.Click
        Me.m_game.BeginGame()
    End Sub

    Private Sub markerToolStripItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles markerToolStripItem.Click
        Me.markerToolStripItem.Checked = Not Me.markerToolStripItem.Checked
        Me.m_game.UseMarker = Me.markerToolStripItem.Checked
    End Sub

    Private Sub MainDialog_FormClosing(ByVal sender As Object, ByVal e As FormClosingEventArgs) Handles Me.FormClosing
        My.Settings.UseMarker = Me.m_game.UseMarker

        Select Case True
            Case Me.beginnerToolStripItem.Checked
                My.Settings.LastGameType = "Beginner"
            Case Me.advancedToolStripItem.Checked
                My.Settings.LastGameType = "Advanced"
            Case Me.expertToolStripItem.Checked
                My.Settings.LastGameType = "Expert"
            Case Me.customToolStripItem.Checked
                My.Settings.LastGameType = "Custom"
                My.Settings.LastGameWidth = Me.m_game.Width
                My.Settings.LastGameHeight = Me.m_game.Height
                My.Settings.LastGameMines = Me.m_game.Mines
        End Select

        My.Settings.Save()
    End Sub

    Private Sub MainDialog_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        Me.aboutToolStripMenuItem.Text = Me.aboutToolStripMenuItem.Text.Replace("...", "")
        Me.customToolStripItem.Text = Me.customToolStripItem.Text.Replace("...", "")
        Me.highscoreToolStripItem.Text = Me.highscoreToolStripItem.Text.Replace("...", "")
        ' TODO: resolve.
        'Me.markerToolStripItem.Text = "&Marker (" & Chr(&H201C) & "?" & Chr(&H201D) & ")"

        Me.m_game = New MinesweeperGame(Me.playField, Me.gameTimer)

        Me.markerToolStripItem.Checked = My.Settings.UseMarker
        Me.m_game.UseMarker = My.Settings.UseMarker

        Select Case My.Settings.LastGameType.ToLower()
            Case "beginner"
                Me.m_game.BeginGame(MinesweeperGame.FixedConfigurations.Beginner)
                ShowGameTypeInMenu(Me.beginnerToolStripItem)
            Case "advanced"
                Me.m_game.BeginGame(MinesweeperGame.FixedConfigurations.Advanced)
                ShowGameTypeInMenu(Me.advancedToolStripItem)
            Case "expert"
                Me.m_game.BeginGame(MinesweeperGame.FixedConfigurations.Expert)
                ShowGameTypeInMenu(Me.expertToolStripItem)
            Case "custom"
                Me.m_game.BeginGame( _
                    My.Settings.LastGameWidth, _
                    My.Settings.LastGameHeight, _
                    My.Settings.LastGameMines _
                )
                ShowGameTypeInMenu(Me.customToolStripItem)
            Case Else
                Me.m_game.BeginGame()
        End Select
    End Sub

    Private Sub pauseToolStripItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles pauseToolStripItem.Click
        Me.m_game.Pause()
    End Sub

    Private Sub highscoreToolStripItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles highscoreToolStripItem.Click
        HighscoreDialog.ShowDialog()
    End Sub

    Private Sub customToolStripItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles customToolStripItem.Click
        Dim customGame As New CustomGameDialog(Me.m_game.Width, Me.m_game.Height, Me.m_game.Mines)

        If customGame.ShowDialog() = Windows.Forms.DialogResult.OK Then
            Me.m_game.BeginGame(customGame.FieldWidth, customGame.FieldHeight, customGame.Mines)
            ShowGameTypeInMenu(sender)
        End If
    End Sub

    Private Sub rulesToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles rulesToolStripMenuItem.Click
        Dim sysDir As String = Environment.GetEnvironmentVariable("windir")

        Try
            Process.Start(sysDir & "\Help\winmine.chm")
        Catch ex As Exception
            MsgBox("Unable to display minesweeper help.", MsgBoxStyle.Critical, "Error")
        End Try
    End Sub

End Class
